@@ -842,9 +842,13 @@ labels = TypeDict({
     'ResampleDialog.reset'         : 'Reset',
     'ResampleDialog.cancel'        : 'Cancel',
     'ResampleDialog.interpolation' : 'Interpolation',
+    'ResampleDialog.origin'        : 'Origin',
     'ResampleDialog.smoothing'     : 'Smoothing',
     'ResampleDialog.allVolumes'    : 'All volumes',
     'ResampleDialog.dtype'         : 'Data type',
+    'ResampleDialog.noref'         : 'None',
+    'ResampleDialog.corner'        : 'Corner',
+    'ResampleDialog.centre'        : 'Centre',
     'ResampleDialog.nearest'       : 'Nearest neighbour',
     'ResampleDialog.linear'        : 'Linear',
     'ResampleDialog.cubic'         : 'Cubic',
@@ -853,6 +857,7 @@ labels = TypeDict({
     'ResampleDialog.sint'          : 'Signed int',
     'ResampleDialog.float'         : 'Float',
     'ResampleDialog.double'        : 'Double',
+    'ResampleDialog.reference'     : 'Resample to reference',
     'ResampleDialog.origVoxels'    : 'Old shape',
     'ResampleDialog.origPixdims'   : 'Old pixdims',
     'ResampleDialog.newVoxels'     : 'New shape',
